/// <reference types="node" />
/// <reference types="chai" />
import { API as StorybookAPI } from '@storybook/api';
import { DecoratorFunction } from '@storybook/addons';
import { IKey } from 'selenium-webdriver/lib/input';
import { Worker as ClusterWorker } from 'cluster';
import { WebDriver, WebElementPromise } from 'selenium-webdriver';
import Pixelmatch from 'pixelmatch';
import { Context } from 'mocha';
export declare type DiffOptions = typeof Pixelmatch extends (x1: any, x2: any, x3: any, x4: any, x5: any, options?: infer T) => void ? T : never;
export declare type SetStoriesData = {
    globalParameters: {
        creevey?: CreeveyStoryParams;
    };
    kindParameters: Partial<{
        [kind: string]: {
            fileName: string;
            creevey?: CreeveyStoryParams;
        };
    }>;
    stories: StoriesRaw;
};
export declare type StoriesRaw = StorybookAPI extends {
    setStories: (stories: infer SS) => void;
} ? SS : never;
export declare type StoryInput = StoriesRaw extends {
    [id: string]: infer S;
} ? S : never;
export interface StoryMeta<StoryFnReturnType = unknown> {
    title: string;
    component?: unknown;
    decorators?: DecoratorFunction<StoryFnReturnType>[];
    parameters?: {
        creevey?: CreeveyStoryParams;
        [name: string]: unknown;
    };
}
export interface CreeveyMeta {
    parameters?: {
        creevey?: CreeveyStoryParams;
        [name: string]: unknown;
    };
}
export interface CSFStory<StoryFnReturnType = unknown> {
    (): StoryFnReturnType;
    /**
     * @deprecated
     * CSF .story annotations deprecated; annotate story functions directly:
     * - StoryFn.story.name => StoryFn.storyName
     * - StoryFn.story.(parameters|decorators) => StoryFn.(parameters|decorators)
     * See https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#hoisted-csf-annotations for details and codemod.
     */
    story?: {
        name?: string;
        decorators?: DecoratorFunction<StoryFnReturnType>[];
        parameters?: {
            creevey?: CreeveyStoryParams;
            [name: string]: unknown;
        };
    };
    storyName?: string;
    decorators?: DecoratorFunction<StoryFnReturnType>[];
    parameters?: {
        creevey?: CreeveyStoryParams;
        [name: string]: unknown;
    };
}
export interface CreeveyStory {
    parameters?: {
        creevey?: CreeveyStoryParams;
        [name: string]: unknown;
    };
}
export interface Capabilities {
    browserName: string;
    version?: string;
}
export declare type BrowserConfig = Capabilities & {
    limit?: number;
    gridUrl?: string;
    storybookUrl?: string;
    /**
     * Specify custom docker image
     * @default `selenoid/${browserName}:${version}`
     */
    dockerImage?: string;
    viewport?: {
        width: number;
        height: number;
    };
};
export declare type Browser = boolean | string | BrowserConfig;
export interface HookConfig {
    before?: () => unknown;
    after?: () => unknown;
}
export interface Config {
    /**
     * Url to Selenium grid hub or standalone selenium.
     * By default creevey will use docker containers
     */
    gridUrl: string;
    /**
     * Url where storybook hosted on
     * @default http://localhost:6006
     */
    storybookUrl: string;
    /**
     * Absolute path to directory with reference images
     * @default path.join(process.cwd(), './images')
     */
    screenDir: string;
    /**
     * Absolute path where test reports and diff images would be saved
     * @default path.join(process.cwd(), './report')
     */
    reportDir: string;
    /**
     * Absolute path to storybook config directory
     * @default path.join(process.cwd(), './.storybook')
     */
    storybookDir: string;
    /**
     * How much test would be retried
     * @default 0
     */
    maxRetries: number;
    /**
     * Define pixelmatch diff options
     * @default { threshold: 0, includeAA: true }
     */
    diffOptions: DiffOptions;
    /**
     * Browser capabilities
     * @default { chrome: true }
     */
    browsers: {
        [key: string]: Browser;
    };
    /**
     * Hooks that allow run custom script before and after creevey start
     */
    hooks: HookConfig;
}
export declare type CreeveyConfig = Partial<Config>;
export interface Options {
    config?: string;
    port: number;
    ui: boolean;
    update: boolean;
    webpack: boolean;
    debug: boolean;
    browser?: string;
    reporter?: string;
    screenDir?: string;
    reportDir?: string;
    saveReport?: boolean;
}
export declare type WorkerMessage = {
    type: 'ready';
    payload?: never;
} | {
    type: 'error';
    payload: {
        error: string;
    };
};
export declare type TestMessage = {
    type: 'start';
    payload: {
        id: string;
        path: string[];
        retries: number;
    };
} | {
    type: 'end';
    payload: TestResult;
};
export declare type WebpackMessage = {
    type: 'success';
    payload?: never;
} | {
    type: 'fail';
    payload?: never;
} | {
    type: 'rebuild succeeded';
    payload?: never;
} | {
    type: 'rebuild failed';
    payload?: never;
};
export declare type DockerMessage = {
    type: 'start';
    payload?: never;
} | {
    type: 'success';
    payload: {
        gridUrl: string;
    };
};
export declare type ShutdownMessage = unknown;
export declare type ProcessMessage = (WorkerMessage & {
    scope: 'worker';
}) | (TestMessage & {
    scope: 'test';
}) | (WebpackMessage & {
    scope: 'webpack';
}) | (DockerMessage & {
    scope: 'docker';
}) | (ShutdownMessage & {
    scope: 'shutdown';
});
export declare type WorkerHandler = (message: WorkerMessage) => void;
export declare type TestHandler = (message: TestMessage) => void;
export declare type WebpackHandler = (message: WebpackMessage) => void;
export declare type DockerHandler = (message: DockerMessage) => void;
export declare type ShutdownHandler = (message: ShutdownMessage) => void;
export interface Worker extends ClusterWorker {
    isRunning?: boolean;
}
export interface Images {
    actual: string;
    expect?: string;
    diff?: string;
    error?: string;
}
export declare type TestStatus = 'unknown' | 'pending' | 'running' | 'failed' | 'success';
export interface TestResult {
    status: 'failed' | 'success';
    images?: Partial<{
        [name: string]: Images;
    }>;
    error?: string;
}
export interface ImagesError extends Error {
    images: string | Partial<{
        [name: string]: string;
    }>;
}
export interface Test {
    id: string;
    path: string[];
    skip: boolean | string;
    retries?: number;
    status?: TestStatus;
    results?: TestResult[];
    approved?: Partial<{
        [image: string]: number;
    }>;
    storyId?: string;
}
export interface ServerTest extends Test {
    story: StoryInput;
    fn: (this: Context) => Promise<void>;
}
export interface CreeveyStatus {
    isRunning: boolean;
    tests: Partial<{
        [id: string]: Test;
    }>;
}
export interface CreeveyUpdate {
    isRunning?: boolean;
    tests?: Partial<{
        [id: string]: Partial<Test> & {
            path: string[];
        };
    }>;
    removedTests?: string[][];
}
export interface SkipOption {
    reason?: string;
    in?: string | string[] | RegExp;
    kinds?: string | string[] | RegExp;
    stories?: string | string[] | RegExp;
    tests?: string | string[] | RegExp;
}
export declare type SkipOptions = string | SkipOption | SkipOption[];
export declare type CreeveyTestFunction = (this: {
    browser: WebDriver;
    keys: IKey;
    expect: Chai.ExpectStatic;
    takeScreenshot: () => Promise<string>;
    readonly captureElement?: WebElementPromise;
}) => Promise<void>;
export interface CreeveyStoryParams {
    captureElement?: string | null;
    delay?: number;
    skip?: SkipOptions;
    tests?: {
        [name: string]: CreeveyTestFunction;
    };
}
export interface ApprovePayload {
    id: string;
    retry: number;
    image: string;
}
export declare type Request = {
    type: 'status';
} | {
    type: 'start';
    payload: string[];
} | {
    type: 'stop';
} | {
    type: 'approve';
    payload: ApprovePayload;
};
export declare type Response = {
    type: 'status';
    payload: CreeveyStatus;
} | {
    type: 'update';
    payload: CreeveyUpdate;
};
export interface CreeveyTest extends Test {
    checked: boolean;
}
export interface CreeveySuite {
    path: string[];
    skip: boolean;
    status?: TestStatus;
    opened: boolean;
    checked: boolean;
    indeterminate: boolean;
    children: Partial<{
        [title: string]: CreeveySuite | CreeveyTest;
    }>;
}
export declare type ImagesViewMode = 'side-by-side' | 'swap' | 'slide' | 'blend';
export declare function noop(): void;
export declare function isDefined<T>(value: T | null | undefined): value is T;
export declare function isTest<T1, T2 extends Test>(x?: T1 | T2): x is T2;
export declare function isObject(x: unknown): x is Record<string, unknown>;
export declare function isString(x: unknown): x is string;
export declare function isImageError(error: unknown): error is ImagesError;
export declare function isProcessMessage(message: unknown): message is ProcessMessage;
export declare function isWorkerMessage(message: unknown): message is WorkerMessage;
export declare function isTestMessage(message: unknown): message is TestMessage;
export declare function isWebpackMessage(message: unknown): message is WebpackMessage;
export declare function isDockerMessage(message: unknown): message is DockerMessage;
